/*
 *  GTK+ K6300 Image Grabber
 *  Author:  Pierre Bureau for K-Team SA <pbureau@k-team.com>
 *  Version: 1.0
 *  Date:    19.11.2002
 */

/*  This file is part of K6300 Image Grabber.
 *  (c) 2002 Pierre Bureau for K-Team SA <pbureau@k-team.com>
 *
 *  K6300 Image Grabber is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *  K6300 Image Grabber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with K6300 Image Grabber; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

#include <gtk/gtk.h>
#include <glib.h>

#ifdef __WIN32
#include <basetyps.h>
#include <basetsd.h>
#include <windef.h>
#include <w32api.h>
#include <winbase.h>
#endif

#include "kt_k6300.h"
#include "kt_serial.h"

#define APPTITLE "K6300 Image Grabber v1.0"
#define MAXRETRY 5

#ifdef __WIN32
const char * portlist[]={"COM1","COM2","COM3","COM4"};
#else
const char * portlist[]={"/dev/ttyS0","/dev/ttyS1","/dev/ttyS2","/dev/ttyS3"};
#endif

const char GPL_text[]="(c) 2002 K-Team SA <pbureau@k-team.com>\n\n\
K6300 Image Grabber is free software; you can redistribute it and/or\n\
modify it under the terms of the GNU General Public License as published\n\
by the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\n\
K6300 Image Grabber is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\n\
You should have received a copy of the GNU General Public License\n\
along with K6300 Image Grabber; if not, write to the Free Software\n\
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n\
This program source code is available from www.k-team.com";

/* Globals for gtk */
GdkPixmap *pixmap;
GdkBitmap *bitmap;
GtkWidget *display;
GtkWidget *mainWin;
GtkWidget *status_bar;
unsigned context;

GdkGC     *gc;

K6300_Image image;

GtkAdjustment *rSet;
GtkAdjustment *gSet;
GtkAdjustment *bSet;


/****************************************************************************************************/
typedef enum SerialStatus_t {idle, waitQ, waitX, starting, downloading} SerialStatus_t;
SerialStatus_t serialStatus=idle;
unsigned timeout=0;

gint serial_comm(gpointer data)
{
    char read[10];
    int status;
    int count;
    static char format[256];
    RGB_Image rgb_image;

    switch(serialStatus)
    {
    case idle:
        break;

    case starting:
        fprintf(stderr,"start\n");
        kt_serial_flush();

        status= kt_command("Q");
        if(status < 0)
        {
            gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,
                               " Turret not responding. Check Connection to serial port.");
            serialStatus= idle;
        }
        else
            serialStatus= waitQ;
        break;

    case waitQ:
        fprintf(stderr,"waitQ\n");
        status= kt_read_buffer(read,1);
        if(status)
            if(strncmp("q",read,1))
            {
                gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
                gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,
                                   " Wrong answer. Reset turret and try again.");
                serialStatus= idle;
            }
            else
            {
                /* Read the "\r\n" that should be hanging in the buffer */
                status= kt_read_buffer(read,2);
                status= kt_command("X");
                if(status < 0)
                {
                    gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
                    gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,
                                       " Turret not responding. Check Connection to serial port.");
                    serialStatus= idle;
                }
                else
                {
                    serialStatus= waitX;
                    format[0]='\0';
                    count=0;
                }
            }
        else
            timeout++;

        if(timeout > MAXRETRY)
        {
            timeout     = 0;
            serialStatus= idle;
            gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,
                               " Turret not responding. Check Connection to serial port.");
        }
        break;

    case waitX:
        fprintf(stderr,"waitX\n");
        gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
        gtk_statusbar_push(GTK_STATUSBAR(status_bar),context," Downloading image...");

        /* Read the image format string */
        status= kt_read_buffer(read,XANSLEN);
        count+= status;
        strncat(format, read, status);
        if(count >= XANSLEN)
        {
            format[XANSLEN]='\0';
            fprintf(stderr,"%s\n",format);
            serialStatus= downloading;
            k6300_start_download(&image,format);
            /* Read the "\n" that should be hanging in the buffer */
            kt_read_buffer(read,1);

        }
        break;

    case downloading:
        fprintf(stderr,"download - %s - %u/%u\n",format,image.download,image.size);

        status= k6300_download(&image);
        if(status == -1)
        {
            gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,
                               " Get image problem. Reset turret and try again.");
            serialStatus= idle;
        }
        if(status == 1)
        {
            gtk_statusbar_pop(GTK_STATUSBAR(status_bar),context);
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context," Download complete");

            k6300_RGB_convert(&image,&rgb_image,rSet->value,gSet->value,bSet->value);
            gdk_draw_rgb_image(pixmap,gc,
                               0,0,rgb_image.w,rgb_image.h,GDK_RGB_DITHER_NORMAL,
                               rgb_image.data,rgb_image.byte_per_line);
            gtk_pixmap_set(GTK_PIXMAP(display), pixmap,NULL);
            gtk_widget_draw(display,NULL);
            serialStatus= idle;
        }
        break;

    }
    return 1;
}

/****************************************************************************************************/

void image_grab(GtkWidget *widget, gpointer data)
{
    if(serialStatus == idle)
        serialStatus= starting;
}

/****************************************************************************************************/
void image_wb(GtkWidget *widget, gpointer data)
{
    RGB_Image rgb_image;

    k6300_white_balance(&image);
    k6300_RGB_convert(&image,&rgb_image,rSet->value,gSet->value,bSet->value);

    gdk_draw_rgb_image(pixmap,gc,
                       0,0,rgb_image.w,rgb_image.h,GDK_RGB_DITHER_NORMAL,
                       rgb_image.data,rgb_image.byte_per_line);

    gtk_pixmap_set(GTK_PIXMAP(display), pixmap,NULL);
    gtk_widget_draw(display,NULL);
}

void imageRGB(GtkAdjustment *widget, gpointer data)
{
    RGB_Image rgb_image;

    k6300_RGB_convert(&image,&rgb_image,rSet->value,gSet->value,bSet->value);

    gdk_draw_rgb_image(pixmap,gc,
                       0,0,rgb_image.w,rgb_image.h,GDK_RGB_DITHER_NORMAL,
                       rgb_image.data,rgb_image.byte_per_line);

    gtk_pixmap_set(GTK_PIXMAP(display), pixmap,NULL);
    gtk_widget_draw(display,NULL);
}

/****************************************************************************************************/
/* Callback for a "close window" event */
gint delete_event( GtkWidget *widget,
                   GdkEvent  *event,
		   gpointer   data )
{
    return FALSE;
}

/* Callback to quit Gtk */
void destroy( GtkWidget *widget, gpointer data )
{                                                                              
    fprintf(stderr,"close\n");
    kt_close_port();

    gtk_main_quit ();
}

/* Callback to close a window */
void destroyWidget( GtkWidget *widget, gpointer data )
{
    gtk_widget_hide(widget);
}
/****************************************************************************************************/
void menu_license(gpointer data,guint action,GtkWidget *menu_item)
{
    GtkWidget *window;
    GtkWidget *box1;
    GtkWidget *text;
    GtkWidget *button;

    window = gtk_window_new(GTK_WINDOW_DIALOG);

    gtk_widget_set_usize (window,500,350);
    gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE);
    gtk_window_set_title (GTK_WINDOW (window), "K6300 Image Grabber License");
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);

    
    box1 = gtk_vbox_new (FALSE, 0);

    text = gtk_text_new (NULL, NULL);
    gtk_text_set_editable (GTK_TEXT(text), FALSE);
    gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
    gtk_box_pack_start(GTK_BOX (box1), text, TRUE, TRUE, 0);
    gtk_widget_show (text);

    gtk_text_insert (GTK_TEXT (text), NULL, &text->style->black, NULL, GPL_text, -1);

    button = gtk_button_new_with_label ("   OK   ");
    gtk_signal_connect_object(GTK_OBJECT (button), "clicked",GTK_SIGNAL_FUNC (destroyWidget),GTK_OBJECT(window));
    gtk_box_pack_start (GTK_BOX (box1), button, FALSE, FALSE, 10);
    gtk_widget_show (button);

    gtk_container_add (GTK_CONTAINER (window), box1);
    gtk_widget_show (box1);

    gtk_widget_show (window);
}

void menu_speed(gpointer data,guint action,GtkWidget *menu_item)
{
    static guint last_speed_action=~0; //Ugly!

    if(action != last_speed_action)
    {
        last_speed_action=action;
        kt_set_serial(action);
    }
}

void menu_port(gpointer data, guint action, GtkWidget *menu_item)
{
    char message[256];
    int status;
    static guint last_port_action=~0;  //Ugly!

    fprintf(stderr,"in\n");
    if(action != last_port_action)
    {
        last_port_action=action;
        kt_close_port();
        status= kt_open_port(portlist[action]);
        if(status == -1)
        {
            sprintf(message," Open Port: cannot open %s",portlist[action]);
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,message);
        } else {
            gtk_statusbar_push(GTK_STATUSBAR(status_bar),context," Press button to grab a new image");
        }
    }
}

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",            NULL,         NULL, 0, "<Branch>" },
  { "/File/License",     NULL,         menu_license,  0, NULL },
  { "/File/Quit",        "<control>Q", gtk_main_quit, 0, NULL },
  { "/Serial _Port",     NULL,         NULL, 0, "<Branch>" },
#ifdef __WIN32
  { "/Serial Port/COM1", NULL,         menu_port, 0, "<RadioItem>" },
  { "/Serial Port/COM2", NULL,         menu_port, 1, "/Serial Port/COM1" },
  { "/Serial Port/COM3", NULL,         menu_port, 2, "/Serial Port/COM1" },
  { "/Serial Port/COM4", NULL,         menu_port, 3, "/Serial Port/COM1" },
#else
  { "/Serial Port/ttyS0",NULL,         menu_port, 0, "<RadioItem>" },
  { "/Serial Port/ttyS1",NULL,         menu_port, 1, "/Serial Port/ttyS0" },
  { "/Serial Port/ttyS2",NULL,         menu_port, 2, "/Serial Port/ttyS0" },
  { "/Serial Port/ttyS3",NULL,         menu_port, 3, "/Serial Port/ttyS0" },
#endif
  { "/_Speed",           NULL,         NULL, 0, "<Branch>" },
  { "/Speed/9600  Baud", NULL,         menu_speed,9600, "<RadioItem>" },
  { "/Speed/19200 Baud", NULL,         menu_speed,19200,"/Speed/9600  Baud"},
  { "/Speed/38400 Baud", NULL,         menu_speed,38400,"/Speed/9600  Baud"}
};

void get_main_menu( GtkWidget  *window, GtkWidget **menubar )
{
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;
    gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

    accel_group = gtk_accel_group_new ();
    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
    gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

    if (menubar)
        *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

/****************************************************************************************************/



int main(int argc, char *argv[])
{
    int status;
    char answer[256];
    RGB_Image rgb_image;

    GdkImage  *test;
    GtkWidget *vbox;
    GtkWidget *mainvbox;
    GtkWidget *hbox;
    GtkWidget *frame;
    GtkWidget *frame2;
    GtkWidget *button;
    GtkWidget *label;
    GtkWidget *hbox2;
    GtkWidget *ruler;
    GtkWidget *menubar;

    char message[256];

    /* Some init */
    gtk_init (&argc, &argv);
    gdk_rgb_init();  

    /* Create Black starting image */
    k6300_create_black_image(&image);
    k6300_RGB_convert(&image,&rgb_image,1.0,1.0,1.0);

    mainWin = gtk_window_new (GTK_WINDOW_DIALOG);
    gtk_signal_connect (GTK_OBJECT (mainWin), "delete_event",
			GTK_SIGNAL_FUNC (delete_event), NULL);
    gtk_signal_connect (GTK_OBJECT (mainWin), "destroy",
                        GTK_SIGNAL_FUNC (destroy), NULL);
    gtk_container_set_border_width (GTK_CONTAINER (mainWin),5);
    gtk_window_set_title (GTK_WINDOW (mainWin), APPTITLE);
    gtk_window_set_default_size (GTK_WINDOW (mainWin),400,260);
    gtk_widget_set_usize (GTK_WIDGET (mainWin),400,260);
    gtk_window_set_policy(GTK_WINDOW(mainWin), FALSE, FALSE, FALSE);
    gtk_widget_show (mainWin);

    mainvbox=gtk_vbox_new(FALSE,5);

    get_main_menu (mainWin, &menubar);
    gtk_box_pack_start(GTK_BOX (mainvbox), menubar, TRUE, TRUE, 0);
    gtk_widget_show(menubar);

    hbox=gtk_hbox_new(FALSE,5);

    /* First Column */
    vbox=gtk_vbox_new(FALSE,5);

    frame = gtk_frame_new(NULL);
    frame2= gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame2),GTK_SHADOW_NONE);
    gtk_container_set_border_width(GTK_CONTAINER(frame2),5);

    gc=mainWin->style->fg_gc[GTK_STATE_NORMAL];

    pixmap=gdk_pixmap_new(mainWin->window,rgb_image.w,rgb_image.h,-1);
    gdk_draw_rgb_image(pixmap,gc,
                       0,0,rgb_image.w,rgb_image.h,GDK_RGB_DITHER_NORMAL,
                       rgb_image.data,rgb_image.byte_per_line);
    display=gtk_pixmap_new(pixmap,NULL);

    gtk_container_add(GTK_CONTAINER(frame2), display);

    gtk_container_add(GTK_CONTAINER(frame),  frame2);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 10);
    gtk_widget_show (display);
    gtk_widget_show (vbox);
    gtk_widget_show (frame);
    gtk_widget_show (frame2);

    /* Second Column */
    vbox=gtk_vbox_new(FALSE,5);

    button = gtk_button_new_with_label ("Grab New Image");
    gtk_signal_connect(GTK_OBJECT (button), "clicked",GTK_SIGNAL_FUNC (image_grab),NULL);
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, TRUE, 2);
    gtk_widget_show (button);

    button = gtk_button_new_with_label ("Compute White Balance");
    gtk_signal_connect(GTK_OBJECT (button), "clicked",GTK_SIGNAL_FUNC (image_wb),NULL);
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, TRUE, 2);
    gtk_widget_show (button);

    hbox2=gtk_hbox_new(FALSE,2);

    label=gtk_label_new("R:");
    gtk_box_pack_start (GTK_BOX (hbox2),label, FALSE, TRUE, 2);
    
    rSet=GTK_ADJUSTMENT(gtk_adjustment_new(1.0,0.1,2.1,0.1,0.5,0.1));
    ruler=gtk_hscale_new(rSet);
    gtk_signal_connect (GTK_OBJECT(rSet), "value_changed",GTK_SIGNAL_FUNC(imageRGB),NULL);
    gtk_scale_set_digits( GTK_SCALE(ruler),1);
    gtk_scale_set_value_pos(GTK_SCALE(ruler),GTK_POS_TOP);
    gtk_box_pack_start (GTK_BOX (hbox2),ruler,TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox),hbox2,FALSE, TRUE, 5);
    gtk_widget_show (ruler);
    gtk_widget_show (label);
    gtk_widget_show (hbox2);

    hbox2=gtk_hbox_new(FALSE,2);

    label=gtk_label_new("G:");
    gtk_box_pack_start (GTK_BOX (hbox2),label, FALSE, TRUE, 2);

    gSet=GTK_ADJUSTMENT(gtk_adjustment_new(1.0,0.1,2.1,0.1,0.5,0.1));
    ruler=gtk_hscale_new(gSet);
    gtk_signal_connect (GTK_OBJECT(gSet), "value_changed",GTK_SIGNAL_FUNC(imageRGB),NULL);
    gtk_scale_set_digits( GTK_SCALE(ruler),1);
    gtk_scale_set_value_pos(GTK_SCALE(ruler),GTK_POS_TOP);
    gtk_box_pack_start (GTK_BOX (hbox2),ruler,TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox),hbox2,FALSE, TRUE, 5);
    gtk_widget_show (ruler);
    gtk_widget_show (label);
    gtk_widget_show (hbox2);

    hbox2=gtk_hbox_new(FALSE,2);

    label=gtk_label_new("B:");
    gtk_box_pack_start (GTK_BOX (hbox2),label, FALSE, TRUE, 2);

    bSet=GTK_ADJUSTMENT(gtk_adjustment_new(1.0,0.1,2.1,0.1,0.5,0.1));
    ruler=gtk_hscale_new(bSet);
    gtk_signal_connect (GTK_OBJECT(bSet), "value_changed",GTK_SIGNAL_FUNC(imageRGB),NULL);
    gtk_scale_set_digits( GTK_SCALE(ruler),1);
    gtk_scale_set_value_pos(GTK_SCALE(ruler),GTK_POS_TOP);
    gtk_box_pack_start (GTK_BOX (hbox2),ruler,TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox),hbox2,FALSE, TRUE, 5);
    gtk_widget_show (ruler);
    gtk_widget_show (label);
    gtk_widget_show (hbox2);

    gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 2);
    gtk_widget_show (vbox);
    gtk_widget_show (hbox);

    gtk_box_pack_start(GTK_BOX (mainvbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show (mainvbox);

    status_bar = gtk_statusbar_new();
    gtk_box_pack_start(GTK_BOX (mainvbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    gtk_container_add (GTK_CONTAINER (mainWin), mainvbox);

    context=gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar),"MessageBar");

    status=kt_open_port(portlist[0]);
    kt_close_port();
    status= kt_open_port(portlist[0]);
    if(status == -1)
    {
        fprintf(stderr,"fail open %s\n",portlist[0]);
        sprintf(message," Open Port: cannot open %s",portlist[0]);
        gtk_statusbar_push(GTK_STATUSBAR(status_bar),context,message);
    } else {
        fprintf(stderr,"open %s\n",portlist[0]);
        gtk_statusbar_push(GTK_STATUSBAR(status_bar),context," Press button to grab a new image");
    }

    gtk_timeout_add(10,serial_comm,NULL);

    gtk_main ();

    return 0;
}
