/*
 *  K6300 C library header
 *  Author:  Pierre Bureau for K-Team SA
 *  Version: 1.0
 *  Date:    19.11.2002
 */

/*  This file is part of K6300 Image Grabber.
 *  (c) 2002 Pierre Bureau for K-Team SA <pbureau@k-team.com>
 *
 *  K6300 Image Grabber is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *  K6300 Image Grabber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with K6300 Image Grabber; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _K6300INC
#define _K6300INC

#ifdef __WIN32
#include <basetyps.h>
#include <basetsd.h>
#include <windef.h>
#include <w32api.h>
//#include <winbase.h>
#else
#include <gtk/gtk.h>
#include <glib.h>
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
//#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#endif

/* Definitions */
typedef enum {binary, ascii, wrong} k6300_Format;
typedef struct K6300_Image
{
    unsigned size;
    unsigned download;
    int dx;
    int dy;
    k6300_Format format;
    float wb_r;
    float wb_b;
    unsigned char * data;
} K6300_Image;

typedef struct RGB_Image
{
    int w;
    int h;
    int byte_per_line;
    unsigned char * data;
}RGB_Image;

extern const char* raw_image_f_1;
extern const char* raw_image_f_2;

/* Exported Functions */
#ifdef __WIN32
int k6300_get_image(HANDLE fd, K6300_Image * image,const char * format);
#else
int k6300_get_image(int fd, K6300_Image * image,const char * format);
#endif

#define XANSLEN __builtin_strlen(raw_image_f_1)

int k6300_create_black_image(K6300_Image * image);
int k6300_invert_image(K6300_Image * image);
int k6300_ascii_to_binary(K6300_Image * image);
int k6300_RGB_convert(K6300_Image * image, RGB_Image * dest,float wr, float wg, float wb);
void k6300_white_balance(K6300_Image * image);

#endif /*_K6300INC*/
