/*                              
 * Khepera Serial communicatin Library
 * Author:  Pierre Bureau for K-Team SA
 * Version: 1.0
 * Date:    19.11.2002
 */

/*  This file is part of K6300 Image Grabber.
 *  (c) 2002 Pierre Bureau for K-Team SA <pbureau@k-team.com>
 *
 *  K6300 Image Grabber is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *  K6300 Image Grabber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with K6300 Image Grabber; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "kt_serial.h"

/* Globals for the serial interface */
#ifdef __WIN32
static HANDLE HdComm;
#else
static int fd;
#endif

#define BUFFER_LENGTH 256

static char buffer[BUFFER_LENGTH];
static char * buffer_ptr;
static unsigned long buffer_len;

#ifdef __WIN32
/*
 * The win32 serial communication interface is still using
 * synchronous read and write to the serial port. This might
 * cause problems when transfering big amount of data.
 */
int kt_open_port(const char * device)
{
    DCB Dcb;
    COMMTIMEOUTS CommTimeOuts;

    HdComm = CreateFile(device,GENERIC_READ|GENERIC_WRITE,0,NULL,
                        OPEN_EXISTING,0,NULL);
    if(HdComm == INVALID_HANDLE_VALUE)
    {
        return -1;
    }
    else
    {
        // Initialize COMM port.
        SetupComm( HdComm, 1024, 1024 );
        GetCommState( HdComm, &Dcb );
        Dcb.BaudRate = CBR_38400;
        Dcb.Parity = NOPARITY;
        Dcb.ByteSize = 8;
        Dcb.StopBits = TWOSTOPBITS;
        Dcb.fAbortOnError = TRUE;
        SetCommState( HdComm, &Dcb );
        
        CommTimeOuts.ReadIntervalTimeout = 100; //120
	CommTimeOuts.ReadTotalTimeoutConstant = 50; //10
	CommTimeOuts.ReadTotalTimeoutMultiplier = 100; //10
	CommTimeOuts.WriteTotalTimeoutConstant = 100; //10
	CommTimeOuts.WriteTotalTimeoutMultiplier = 100; //10
        if(!SetCommTimeouts(HdComm,&CommTimeOuts))
        {
             return -1;
        }
    }

    PurgeComm(HdComm,PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR);

    return 0;
}

int kt_set_serial(unsigned speed)
{
    DCB Dcb;
    COMMTIMEOUTS CommTimeOuts;

    SetupComm( HdComm, 1024, 1024 );
    GetCommState( HdComm, &Dcb );
    /* Set speed */
    switch(speed)
    {
    case 9600 :
        Dcb.BaudRate = CBR_9600;
        break;
    case 19200 :
        Dcb.BaudRate = CBR_19200;
        break;
    case 38400 :
        Dcb.BaudRate = CBR_38400;
        break;
    case 57600 :
        Dcb.BaudRate = CBR_57600;
        break;
    default :
        Dcb.BaudRate = CBR_115200;
        break;
    }

    Dcb.Parity = NOPARITY;
    Dcb.ByteSize = 8;
    Dcb.StopBits = TWOSTOPBITS;
    Dcb.fAbortOnError = TRUE;
    SetCommState( HdComm, &Dcb );

    CommTimeOuts.ReadIntervalTimeout = 100; //120
    CommTimeOuts.ReadTotalTimeoutConstant = 50; //10
    CommTimeOuts.ReadTotalTimeoutMultiplier = 100; //10
    CommTimeOuts.WriteTotalTimeoutConstant = 100; //10
    CommTimeOuts.WriteTotalTimeoutMultiplier = 100; //10
    if(!SetCommTimeouts(HdComm,&CommTimeOuts))
    {
        return -1;
    }

    return 0;
}

void kt_close_port()
{
    CloseHandle(HdComm);
}

void kt_serial_flush()
{
    PurgeComm(HdComm,PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR);
    buffer_len=0;
    buffer_ptr=buffer;
}

int kt_command(const char * cmd)
{
    DWORD nbytes;
    DWORD dwEvtMask;
    ULONG lpCompletionKey;
    OVERLAPPED ov;
    HANDLE event;
    DWORD stat;

    //event= CreateEvent(NULL,FALSE,FALSE,NULL);

    ov.Offset     = 0;
    ov.OffsetHigh = 0;
    ov.hEvent     = event;

    PurgeComm(HdComm,PURGE_RXABORT|PURGE_RXCLEAR);

    if(!WriteFile(HdComm,(LPCVOID)cmd,strlen(cmd),&nbytes,NULL))
    {
        /*stat = GetLastError();
        if(stat == ERROR_IO_PENDING)
        {
            GetOverlappedResult(HdComm,&ov,&nbytes,TRUE);
        }
        else*/
            return -1;
    }

    if(!WriteFile(HdComm,"\n",1,&nbytes,NULL))
    {
        /*stat = GetLastError();
        if(stat == ERROR_IO_PENDING)
        {
            GetOverlappedResult(HdComm,&ov,&nbytes,TRUE);
        }
        else*/
            return -1;
    }

    dwEvtMask=EV_TXEMPTY;
    WaitCommEvent(HdComm,&dwEvtMask,NULL);

    return 0;                 
}

int kt_read_buffer(char * answer, unsigned  maxlen)
{
    DWORD result;
    OVERLAPPED ov;

    ov.Offset     = 0;
    ov.OffsetHigh = 0;
    ov.hEvent     = 0;

    ReadFile(HdComm,(LPVOID)answer,maxlen,&result,NULL);

    return result;
}
#else
/***************************************************************************************************/
/*
 * Definition of signal handler for asynchronous communication
 */
void signal_handler_IO(int status)
{
    int stat;
    char tmpbuf[BUFFER_LENGTH];

//    fprintf(stderr,"received SIGIO signal (0x%lx:%lu) - ",buffer_ptr,buffer_len);
    if(buffer_len < BUFFER_LENGTH)
    {
        stat= read(fd,tmpbuf,BUFFER_LENGTH-buffer_len);
        if(stat > 0)
        {
            tmpbuf[stat]='\0';
            strncpy(buffer_ptr,tmpbuf,stat);
            buffer_len+= stat;
            buffer_ptr+= stat;

//            fprintf(stderr,"%d:%s: buf=%lu\n",stat,tmpbuf,buffer_len);
        }
    }
    else
        /* The storage buffer is full and some characters might be missed */
        fprintf(stderr,"Missing characters!!!\n");
}

/*
 * Open serial port
 * Linux: Returns 0 on success or -1 on error.
 */
int kt_open_port(const char * device)
{
  struct termios oldtio,newtio;
  struct sigaction saio; /* definition of signal action */

  int flags;
  int yesplease = 1;

  fd = open(device, O_RDWR | O_NOCTTY | O_NONBLOCK);
  if (fd == -1)
  {
      fprintf(stderr,"fail open %s\n",device);
      return -1;
  }
  else
  {
      fprintf(stderr,"open %s - %d\n",device,fd);
      /* install the signal handler before making the device asynchronous */
      saio.sa_handler = signal_handler_IO;
      saio.sa_flags = 0;
      sigaction(SIGIO,&saio,NULL);

      flags = fcntl( fd, F_GETFL, 0 );

      /* allow the process to receive SIGIO */
      fcntl(fd, F_SETOWN, getpid());
      /* Make the file descriptor asynchronous (the manual page says only
       O_APPEND and O_NONBLOCK, will work with F_SETFL...) */
      fcntl(fd, F_SETFL, flags | FASYNC);

      tcgetattr(fd,&oldtio); /* save current port settings */
      /* set new port settings for canonical input processing */
      newtio.c_cflag = CS8 | CLOCAL | CREAD | CSTOPB;
      newtio.c_iflag = IGNPAR | ICRNL;
      newtio.c_oflag = 0;
      newtio.c_lflag = 0;
      newtio.c_cc[VMIN]=0;
      newtio.c_cc[VTIME]=0;
                                                                      
      /* Set spped */
      cfsetispeed(&newtio, B38400);
      cfsetospeed(&newtio, B38400);

      tcflush(fd, TCIFLUSH);
      tcsetattr(fd,TCSANOW,&newtio);

      buffer_ptr= buffer;
      buffer_len= 0;
  }

  return 0;
}

/*
 * Set the serial communiation Speed
 */
int kt_set_serial(unsigned speed)
{
    struct termios options;

    /* Get config */
    tcgetattr(fd, &options);

    /* Set spped */
    switch(speed)
    {
    case 9600 :
        cfsetispeed(&options, B9600);
        cfsetospeed(&options, B9600);
        break;
    case 19200 :
        cfsetispeed(&options, B19200);
        cfsetospeed(&options, B19200);
        break;
    case 38400 :
        cfsetispeed(&options, B38400);
        cfsetospeed(&options, B38400);
        break;
    case 57600 :
        cfsetispeed(&options, B57600);
        cfsetospeed(&options, B57600);
        break;
    default :
        cfsetispeed(&options, B115200);
        cfsetospeed(&options, B115200);
        break;
    }

    return 0;
}

void kt_close_port()
{
    close(fd);
}

void kt_serial_flush()
{
    tcflush(fd, TCIOFLUSH);
    buffer_len=0;
    buffer_ptr=buffer;
}

/*
 * Send the given command to the serial line, adding a final '\n'.
 */
int kt_command(const char * cmd)
{
    int stat;

    tcflush(fd,TCIFLUSH);

    stat= write(fd,cmd,strlen(cmd));
    if(stat < 0)
        return -1;
    stat= write(fd,"\n",1);
    if(stat < 0)
        return -1;

    tcdrain(fd);

    return 0;
}

int kt_read_buffer(char * answer, unsigned  maxlen)
{
    unsigned cpylen;
    sigset_t set;

    sigemptyset(&set);
    sigaddset(&set, SIGIO);
    sigprocmask(SIG_BLOCK, &set, NULL);

    if(maxlen > buffer_len)
        cpylen= buffer_len;
    else
        cpylen= maxlen;

    strncpy(answer,buffer,cpylen);

    if(cpylen)
    {
        /* update buffer */
        strncpy(buffer,buffer+cpylen,buffer_len-cpylen);
        buffer_len-= cpylen;
        buffer_ptr-= cpylen;
    }

    sigprocmask(SIG_UNBLOCK, &set, NULL);

    return (int)cpylen;
}

#endif
